// get localisation - language support
var text;
var json_url =  jQuery('#strings').prop('src');


jQuery.ajax({
	  dataType: "json",
	  url: json_url,
	  data: null,
	  async: false,
	  success: function (data) { text = data;  }
	});

String.prototype.format = String.prototype.f = function() {
    var s = this,
        i = arguments.length;

    while (i--) {
        s = s.replace(new RegExp('\\{' + i + '\\}', 'gm'), arguments[i]);
    }
    return s;
};


// board.js
var DEBUG = false;
var withAI = true;

var img_dir =  jQuery('#images').attr('dir');
if (img_dir == undefined || img_dir == null)
  img_dir = 'images';	



var imgs = [null, new Image(), new Image(), new Image(), new Image()];
imgs[1].src = img_dir + '/r.png';
imgs[2].src = img_dir + '/w.png';
imgs[3].src = img_dir + '/r_super.png';
imgs[4].src = img_dir + '/w_super.png';

var PLAYER = {P1: 1, P2: 2};

var PIECE_TYPE = {
    NO_PIECE: 0,
    RED_PIECE: 1,
    YELLOW_PIECE: 2,
    SUPER_RED: 3,
    SUPER_YELLOW: 4
};

var SQR = {
    A1:7, A2:13, A3:19, A4:25, A5:31, A6:37, A7:43, A8:49,
    B1:8, B2:14, B3:20, B4:26, B5:32, B6:38, B7:44, B8:50,
    C1:9, C2:15, C3:21, C4:27, C5:33, C6:39, C7:45, C8:51,
    D1:10, D2:16, D3:22, D4:28, D5:34, D6:40, D7:46, D8:52,
    NONE: 0
};

var FILE = {
    FILE_A:0, FILE_B:1, FILE_C:2, FILE_D:3, FILE_NONE: 4
};

var RANK = {
    RANK_1:0, RANK_2:1, RANK_3:2, RANK_4:3,
    RANK_5:4, RANK_6:5, RANK_7:6, RANK_8:7,
    RANK_NONE: 9
};

var OFF_BOARD = [
    0,1,2,3,4,5,
    6,12,18,24,30,36,42,48,54,
    11,17,23,29,35,41,47,53,59,
    55,56,57,58
];

var EVEN_RANK_SQR = [
    SQR.A2, SQR.B2, SQR.C2, SQR.D2, SQR.A4, SQR.B4, SQR.C4, SQR.D4,
    SQR.A6, SQR.B6, SQR.C6, SQR.D6, SQR.A8, SQR.B8, SQR.C8, SQR.D8
];

var ODD_RANK_SQR = [
    SQR.A1, SQR.B1, SQR.C1, SQR.D1, SQR.A3, SQR.B3, SQR.C3, SQR.D3,
    SQR.A5, SQR.B5, SQR.C5, SQR.D5, SQR.A7, SQR.B7, SQR.C7, SQR.D7
];

var SIDE_SQR = [SQR.A2, SQR.A4, SQR.A6, SQR.A8, SQR.D1, SQR.D3, SQR.D5, SQR.D7];

var BOARD_SIZE = 60;
var   BOARD_DEF = {};

var sqr48 = [];

BOARD_DEF.board = new Array(BOARD_SIZE);
BOARD_DEF.move = PLAYER.P2;
BOARD_DEF.availableMoves = null;

BOARD_DEF.rPieces = [
    SQR.A1, SQR.B1, SQR.C1, SQR.D1,
    SQR.A2, SQR.B2, SQR.C2, SQR.D2,
    SQR.A3, SQR.B3, SQR.C3, SQR.D3
];

BOARD_DEF.yPieces = [
    SQR.A8, SQR.B8, SQR.C8, SQR.D8,
    SQR.A7, SQR.B7, SQR.C7, SQR.D7,
    SQR.A6, SQR.B6, SQR.C6, SQR.D6
];

var MOVE_TYPE = {
    MOVE_NORMAL : 0,
    MOVE_CAPTURE : 1
};

var SUPER_SQR = [[SQR.A8, SQR.B8, SQR.C8, SQR.D8], [SQR.A1, SQR.B1, SQR.C1, SQR.D1]];

function frToSqr(f, r){
    return (7 + f) + (r * 6);
}

function initBoard(bf){
    for(var i=0; i<BOARD_SIZE; i++){
        bf.board[i] = PIECE_TYPE.NO_PIECE;
    }

    for(var r=RANK.RANK_1; r<Object.keys(RANK).length-1; r++){
        for(var f=FILE.FILE_A; f<Object.keys(FILE).length-1; f++){
            sqr48.push(frToSqr(f, r));
        }
    }

    Object.keys(SQR).forEach(function(key) {
        if(hasPieceAt(bf.rPieces, SQR[key])){
            bf.board[SQR[key]] = PIECE_TYPE.RED_PIECE;

        }
        if(hasPieceAt(bf.yPieces, SQR[key])){
            bf.board[SQR[key]] = PIECE_TYPE.YELLOW_PIECE;
        }
    });

    bf.availableMoves = generateMove(bf.move, bf);
}

function movePiece(src, target, bf){
    if(bf.board[src] != PIECE_TYPE.NO_PIECE && bf.board[target] == PIECE_TYPE.NO_PIECE){
        for(var m of bf.availableMoves){
            if(m.piece == src && m.moves.includes(target)){
                insertPiece(src, target, bf);
                switchPlayer(bf);
                return [true, MOVE_TYPE.MOVE_NORMAL];
            }else if(m.piece == src && m.captures.length > 0){
                for(var capture of m.captures){
                    if(target == capture.to){
                        removePiece(capture.remove, bf);
                        insertPiece(src, target, bf);

                        if(!hasMultipleCapture(target, bf)){
                            switchPlayer(bf);
                        }

                        return [true, MOVE_TYPE.MOVE_CAPTURE, capture.remove];
                    }
                }
            }
        }
    }
    return [false];
}

function removePiece(sqr, bf){
    if(bf.board[sqr] == PLAYER.P1){
        bf.rPieces.splice(bf.rPieces.indexOf(sqr), 1);
    }else if(bf.board[sqr] == PLAYER.P2){
        bf.yPieces.splice(bf.yPieces.indexOf(sqr), 1);
    }else if(PIECE_TYPE.SUPER_RED == bf.board[sqr]){
        bf.rPieces.splice(bf.rPieces.indexOf(sqr), 1);
    }else if(PIECE_TYPE.SUPER_YELLOW == bf.board[sqr]){
        bf.yPieces.splice(bf.yPieces.indexOf(sqr), 1);
    }

    bf.board[sqr] = PIECE_TYPE.NO_PIECE;
}

function insertPiece(src, target, bf){
    bf.board[target] = bf.board[src];
    bf.board[src] = PIECE_TYPE.NO_PIECE;
    if(PIECE_TYPE.RED_PIECE == bf.board[target]){
        bf.rPieces[bf.rPieces.indexOf(src)] = target;
    }else if(PIECE_TYPE.YELLOW_PIECE == bf.board[target]){
        bf.yPieces[bf.yPieces.indexOf(src)] = target;
    }else if(PIECE_TYPE.SUPER_RED == bf.board[target]){
        bf.rPieces[bf.rPieces.indexOf(src)] = target;
    }else if(PIECE_TYPE.SUPER_YELLOW == bf.board[target]){
        bf.yPieces[bf.yPieces.indexOf(src)] = target;
    }
}

function upgradePiece(bf){
    for(var i=0; i<bf.rPieces.length; i++){
        if(SUPER_SQR[0].includes(bf.rPieces[i])){
            bf.board[bf.rPieces[i]] = PIECE_TYPE.SUPER_RED;
        }
    }

    for(var i=0; i<bf.yPieces.length; i++){
        if(SUPER_SQR[1].includes(bf.yPieces[i])){
            bf.board[bf.yPieces[i]] = PIECE_TYPE.SUPER_YELLOW;
        }
    }
}

function switchPlayer(bf){

    upgradePiece(bf);

    bf.move = bf.move == PLAYER.P1 ? PLAYER.P2 : PLAYER.P1;
    bf.availableMoves = generateMove(bf.move, bf);
}

function hasMultipleCapture(target, bf){

    var moves = generateMove(bf.move, bf);

    for(var move of moves){
        if(move.captures.length > 0 && move.piece == target){
            bf.availableMoves = moves.filter(pp => pp.piece == target);
            return true;
        }
    }

    return false;
}

function generateMove(player, bf){
    var possibleMoves = [];
    var pieceCount = 0;
    var pieces = [];

    if(player == PLAYER.P1){
        pieceCount = bf.rPieces.length;
        pieces = bf.rPieces;
    }else if(player == PLAYER.P2){
        pieceCount = bf.yPieces.length;
        pieces = bf.yPieces;
    }

    var hasCaptureMove = false;
    var superCap = false;

    for(var i=0; i<pieceCount; i++){
        var moves = moveCount(player, pieces[i]);
        var m = {piece: pieces[i], moves: [], captures: []};

        var move1 = pieces[i] + moves[0];
        var move2 = pieces[i] + moves[1];
        var move3 = pieces[i] + moves[2];
        var move4 = pieces[i] + moves[3];

        if (bf.board[pieces[i]] == PIECE_TYPE.SUPER_YELLOW || bf.board[pieces[i]] == PIECE_TYPE.SUPER_RED)
        {
            superCap = superPieceCapture(m, pieces[i], bf) ? true : superCap;
            hasCaptureMove = hasCaptureMove ? true : superCap;
        }

        hasCaptureMove = checkCaptureMoves(m, move1, player, 0, bf) ? true : hasCaptureMove;
        hasCaptureMove = checkCaptureMoves(m, move2, player, 1, bf) ? true : hasCaptureMove;
        hasCaptureMove = checkCaptureMoves(m, move3, player, 2, bf) ? true : hasCaptureMove;
        hasCaptureMove = checkCaptureMoves(m, move4, player, 3, bf) ? true : hasCaptureMove;

        if(m.captures.length > 0)
            possibleMoves.push(m)
    }

    if(!hasCaptureMove){
        for(var i=0; i<pieceCount; i++){
            var moves = moveCount(player, pieces[i]);
            var m = {piece: pieces[i], moves: [], captures: []};
            var move1 = pieces[i] + moves[0];
            var move2 = pieces[i] + moves[1];

            if(bf.board[pieces[i]] == PIECE_TYPE.SUPER_RED || bf.board[pieces[i]] == PIECE_TYPE.SUPER_YELLOW){
                superPieceMove(m, pieces[i], bf);
                if(m.moves.length > 0) possibleMoves.push(m);
                continue;
            }

            if(bf.board[move1] == PIECE_TYPE.NO_PIECE && !inOffset(move1)){
                m.moves.push(pieces[i] + moves[0]);
            }
            if(bf.board[move2] == PIECE_TYPE.NO_PIECE && !inOffset(move2)){
                m.moves.push(pieces[i] + moves[1]);
            }

            if(m.moves.length > 0) possibleMoves.push(m);
        }
    }
    return possibleMoves;
}

function superPieceCapture(piece, sqr, bf){
    var mUpRight = sqr + sqrPieceEvenOrOdd(sqr)[0];
    var mUpLeft = sqr + sqrPieceEvenOrOdd(sqr)[1];
    var mDownLeft= sqr +  sqrPieceEvenOrOdd(sqr)[2];
    var mDownRight = sqr +  sqrPieceEvenOrOdd(sqr)[3];

    var hasCap = false;
    hasCap = sPieceCap(mUpRight, 0, piece, bf) ? true : hasCap;
    hasCap = sPieceCap(mUpLeft, 1, piece, bf) ? true : hasCap;
    hasCap = sPieceCap(mDownLeft, 2, piece, bf) ? true : hasCap;
    hasCap = sPieceCap(mDownRight, 3, piece, bf) ? true : hasCap;

    return hasCap;
}

function sPieceCap(direct, i, piece, bf){
    var hasCap = false;
    while(!inOffset(direct)){
        if(bf.board[direct] != PIECE_TYPE.NO_PIECE){
            var sPieceType = bf.move == PLAYER.P1 ? PIECE_TYPE.SUPER_RED : PIECE_TYPE.SUPER_YELLOW;

            if(bf.board[direct] == bf.move || bf.board[direct] == sPieceType) break;

            var toRemove = direct;

            direct += sqrPieceEvenOrOdd(direct)[i];
            if(bf.board[direct] == PIECE_TYPE.NO_PIECE && !inOffset(direct)){
                piece.captures.push({to: direct, remove: toRemove});
                hasCap = true;
            }

            break;

        }
        direct += sqrPieceEvenOrOdd(direct)[i];
    }

    return hasCap;
}

function superPieceMove(piece, sqr, bf){

    var mUpRight = sqr + sqrPieceEvenOrOdd(sqr)[0];
    var mUpLeft = sqr + sqrPieceEvenOrOdd(sqr)[1];
    var mDownLeft= sqr +  sqrPieceEvenOrOdd(sqr)[2];
    var mDownRight = sqr +  sqrPieceEvenOrOdd(sqr)[3];

    while(bf.board[mUpRight] == PIECE_TYPE.NO_PIECE && !inOffset(mUpRight)){
        piece.moves.push(mUpRight);
        mUpRight += sqrPieceEvenOrOdd(mUpRight)[0];
    }

    while(bf.board[mUpLeft] == PIECE_TYPE.NO_PIECE && !inOffset(mUpLeft)){
        piece.moves.push(mUpLeft);
        mUpLeft += sqrPieceEvenOrOdd(mUpLeft)[1];
    }

    while(bf.board[mDownLeft] == PIECE_TYPE.NO_PIECE && !inOffset(mDownLeft)){
        piece.moves.push(mDownLeft);
        mDownLeft += sqrPieceEvenOrOdd(mDownLeft)[2];
    }

    while(bf.board[mDownRight] == PIECE_TYPE.NO_PIECE && !inOffset(mDownRight)){
        piece.moves.push(mDownRight);
        mDownRight += sqrPieceEvenOrOdd(mDownRight)[3];
    }

}

function sqrPieceEvenOrOdd(sqr){
    if(EVEN_RANK_SQR.includes(sqr))
        return [-6, -7, 5, 6];

    return[-5, -6, 6, 7];
}

function checkCaptureMoves(m, move, player, tmove, bf){
    if(bf.board[move] != PIECE_TYPE.NO_PIECE && player != bf.board[move] && !inOffset(move)){
        var moveTemp = moveCount(player, move);

        var sPieceType = player == PLAYER.P1 ? PIECE_TYPE.SUPER_RED : PIECE_TYPE.SUPER_YELLOW;

        if(bf.board[move] == sPieceType)
            return false;

        if(bf.board[move + moveTemp[tmove]] == PIECE_TYPE.NO_PIECE  && !inOffset(move + moveTemp[tmove])){
            m.captures.push({to: move + moveTemp[tmove], remove: move});
            return true;
        }
    }
    return false;
}

function moveCount(player, sqr){
    if(player == PLAYER.P1){
        if(EVEN_RANK_SQR.includes(sqr)){
            return [5, 6, -6, -7];
        }else if(ODD_RANK_SQR.includes(sqr)){
            return [6, 7, -5, -6];
        }
    }
    else if(player == PLAYER.P2)
    {
        if(EVEN_RANK_SQR.includes(sqr)){
            return [-6, -7, 5, 6];
        }else if(ODD_RANK_SQR.includes(sqr)){
            return [-5, -6, 6, 7];
        }
    }
    return [0,0];
}

function inOffset(pos){
    return OFF_BOARD.includes(pos);
}

function hasPieceAt(pos, index){
    return pos.includes(index);
}

function printBoard(){
    var s = "";
    s += "<center>";
    var x = 0;
    var y = false;
   for(var i=0; i<sqr48.length*2; i++){
        if(i % 8 == 0)
        {
            s += "<br>";
            y = !y;
        }
        if(i % 2 == 0){
            if(y) s += "--";
            s += ""+BOARD_DEF.board[sqr48[x]];
            if(!y) s += "--";
            x++;
        }
    }
   s += "</center>";
   return s;
}

//piece.js
var Piece = class Piece{

    constructor(x, y, size, pType){
        this.x = x;
        this.y = y;
        this.size = size;
        this.pieceType = pType;
    }

    draw(context){
        context.drawImage(imgs[this.pieceType], this.x, this.y, this.size, this.size);
    }

    mouseMove(mp){
       	this.size = mp.size * 1.1;
        this.x = mp.x - this.size /2;
        this.y = mp.y - this.size /2;
    }

    move(src, target, tiles){
        return movePiece(src, target, BOARD_DEF);
    }
}

//tile.js
var Tile = class Tile {
    constructor(x, y, size, c, sqr, p=null){
        this.x = x;
        this.y = y;
        this.size = size;
        this.sqr = sqr;
        this.piece = p;
        if(c)
            this.color = 'black';
        else
            this.color = 'white';
    }

    draw(context){
        context.fillStyle = this.color;
        context.fillRect(this.x, this.y, this.size, this.size);
        if(DEBUG){
            if(this.color == 'black'){
            context.fillStyle = "blue";
            context.font = "30px Arial";
            context.fillText(this.sqr, this.x, this.y);
            }
        }
    }

    drawPiece(context){
        if(this.piece != null){
            this.piece.draw(context);
        }
    }
}

//ai.js
var AI = PLAYER.P1;

var AI_LVL = 2;

function aiMove(bf){

    if(bf.availableMoves.length == 0 || bf.move != AI)return;

    var bb = tempBfClone(bf);
    var difficulty = document.getElementById('level');
    if (difficulty == null)
      alert ('id for "level" not found');    
    var level = difficulty.options[difficulty.selectedIndex].value;  
    
    var depth = AI_LVL * level;
    var move = minmax(bb, depth, -Infinity, Infinity, AI)[0];

    var verdict;
    if(move.capture){
        verdict = movePiece(move.src, move.target.to, bf);
    }else{
        verdict = movePiece(move.src, move.target, bf);
    }

    var mpiece = null;
    var index = null;
    for(var i=0; i<tiles.length; i++){
        if(tiles[i].sqr == move.src){
            mpiece = tiles[i];
        }

        if(move.capture){
            if(tiles[i].sqr == move.target.to) index = i;
        }else{
            if(tiles[i].sqr == move.target) index = i;
        }
    }

    updatePiecesPos(mpiece, verdict, index, bf);

}

function minmax(bf, depth, alpha, beta, turn){

    if (depth <= 0 || bf.availableMoves <= 0) return evaluatePosition(bf);
    if(turn == AI){

        var bestScore = -Infinity;
        var bestMove = null;

        var tempBf = tempBfClone(bf);
        for(var pieceMove of tempBf.availableMoves){
            var moves;
            var isCapture = false;

            if(pieceMove.moves.length > 0){
                moves = pieceMove.moves;
            }else{
                moves = pieceMove.captures;
                isCapture = true;
            }

            for(var move of moves){

                createNewNode(tempBf, pieceMove.piece, move, isCapture);
                var value = minmax(tempBf, depth-1, alpha, beta, tempBf.move)[1];

                if (value > bestScore){
                    bestScore = value;
                    bestMove = {src: pieceMove.piece, target: move, capture: isCapture};
                }
                alpha = Math.max(alpha, value);

                tempBf = tempBfClone(bf);

                if(alpha >= beta) break;
            }
            if(alpha >= beta) break;
        }
        return [bestMove, bestScore];

    }else{
        var bestScore = Infinity;
        var bestMove = null;

        var tempBf = tempBfClone(bf);

        for(var pieceMove of tempBf.availableMoves){
            var moves;
            var isCapture = false;

            if(pieceMove.moves.length > 0){
                moves = pieceMove.moves;
            }else{
                moves = pieceMove.captures;
                isCapture = true;
            }

            for(var move of moves){

                createNewNode(tempBf, pieceMove.piece, move, isCapture);
                var value = minmax(tempBf, depth-1, alpha, beta, tempBf.move)[1];

                if (value < bestScore){
                    bestScore = value;
                    bestMove = {src: pieceMove.piece, target: move, capture: isCapture};
                }
                beta = Math.min(beta, value);

                tempBf = tempBfClone(bf);

                if(alpha >= beta) break;
            }
            if(alpha >= beta) break;
        }
        return [bestMove, bestScore];

     }
}

function createNewNode(bf, src, target, isCapture){
    if(isCapture){
        movePiece(src, target.to, bf);
    }else{
        movePiece(src, target, bf);
    }
}

function evaluatePosition(bf){

    var rPieceSum = 0;
    var yPieceSum = 0;

    for(var p of bf.rPieces){
        if(bf.board[p] == PIECE_TYPE.SUPER_RED){
            rPieceSum += 3;
        }else if(bf.board[p] == PLAYER.P1){
            rPieceSum++;
        }
    }

    for(var p of bf.yPieces){
        if(bf.board[p] == PIECE_TYPE.SUPER_YELLOW){
            yPieceSum += 3;
        }else if(bf.board[p] == PLAYER.P2){
            yPieceSum++;
        }
    }

    var eval =  rPieceSum - yPieceSum;
    return [null, eval];
}

function tempBfClone(bf){
    var retBoard = {};
    retBoard.board = new Array(BOARD_SIZE);
    retBoard.yPieces = [];
    retBoard.rPieces = [];
    retBoard.availableMoves = [];
    retBoard.move = bf.move;

    for(var i=0; i<BOARD_SIZE; i++){
        retBoard.board[i] = bf.board[i];
    }

    for(var i=0; i<bf.rPieces.length; i++){
        retBoard.rPieces[i] = bf.rPieces[i];
    }

    for(var i=0; i<bf.yPieces.length; i++){
        retBoard.yPieces[i] = bf.yPieces[i];
    }

    for(var i=0; i<bf.availableMoves.length; i++){
        retBoard.availableMoves[i] = bf.availableMoves[i];
    }

    return retBoard;
}

// main.js
var canvas = document.getElementById("canvas"); 
var context = canvas.getContext('2d');
canvas.style.border = '8px double #434323';

if (document.getElementById("lang") != null)
 var lang = document.getElementById("lang").lang;  
else
 var lang = "en"; 

update_status(text[lang].status, 'init');

if (is_touch_enabled())
 jQuery("#hint").html(text[lang].hint1);	  
else
 jQuery("#hint").html(text[lang].hint2); 
 
var tilesize = canvas.width / 8;

var tiles = [];
var stones = [];

var moving = false;
var checker_moves = 0;
var result_draw = false;
var mousepos = 0;
var piecemoving = null;

function init(){

    var colorblack = false;
    var indexOfSqr = 0;
    for(var r=0; r<8; r++){
        for(var c=0; c<8; c++){
            var p = null;
            if(colorblack){
                if(BOARD_DEF.board[sqr48[indexOfSqr]] != PIECE_TYPE.NO_PIECE)
                {
                    p = new Piece(c*tilesize, r*tilesize, tilesize, BOARD_DEF.board[sqr48[indexOfSqr]]);
                }
                tiles.push(new Tile(c*tilesize, r*tilesize, tilesize, colorblack, sqr48[indexOfSqr], p));
                indexOfSqr++;
            }else{
                tiles.push(new Tile(c*tilesize, r*tilesize, tilesize, colorblack, SQR.NONE, p));
            }
            colorblack = !colorblack;
        }
        colorblack = !colorblack;
    }
}

// Set up touch events for mobile tabvars, touch devices
var lastMove = null;

canvas.addEventListener("touchstart", function (e) {
  e.preventDefault();	
  var mousePos = getTouchPos(canvas, e);
  var touch = e.touches[0];
  var mouseEvent = new MouseEvent("mousedown", {
    clientX: touch.clientX,
    clientY: touch.clientY
  });
  canvas.dispatchEvent(mouseEvent);
}, false);

canvas.addEventListener("touchend", function (e) {
  e.preventDefault();  
  var touch = lastMove.touches[0];
  var mouseEvent = new MouseEvent("mouseup", {	
	    clientX: touch.clientX,
	    clientY: touch.clientY  
  });
  canvas.dispatchEvent(mouseEvent);
}, false);

canvas.addEventListener("touchmove", function (e) {
 e.preventDefault();		
  var touch = e.touches[0];
  lastMove = e;
  var mouseEvent = new MouseEvent("mousemove", {
	clientX: touch.clientX,
    clientY: touch.clientY
  });
  canvas.dispatchEvent(mouseEvent);
}, false);

// Get the position of a touch relative to the canvas
function getTouchPos(canvasDom, touchEvent) {
  var rect = canvasDom.getBoundingClientRect();
  return {
    x: touchEvent.touches[0].clientX - rect.left,
    y: touchEvent.touches[0].clientY - rect.top
  };
}

// detect touch device
function is_touch_enabled() { 
	if ("ontouchstart" in document.documentElement)
    return true; 
	 else 
    return false;       
} 

function updateStones () {
	
	var out = "";
	var j = 0;
	var rstone = 0;
	var wstone = 0;
	var r_checker = 0;
	var w_checker = 0
	stones = [];

    for (var i=0; i<tiles.length; i++)
    {
    	if (tiles[[i]].piece != null)
    	{	
    	  stones[j] = tiles[i];  	
    	  j++;
    	  if (tiles[i].piece.pieceType == 1 || tiles[i].piece.pieceType == 3)
    		rstone ++;
    	  else
    		wstone ++; 
    	  
    	  if (tiles[i].piece.pieceType == 3)
    	    r_checker++;  
    	  else if (tiles[i].piece.pieceType == 4)
    	    w_checker++; 
    	}
    }	
    
    
    if (stones.length < 3)    {	       
   	
    	if (w_checker == r_checker && stones.length == 2)
    	  {	  		
    	   	checker_moves ++;    		
	        if (checker_moves >  4)
		       result_draw = true;	     
	   	  }	
    }
   	
    out = text[lang].status2.format(rstone, wstone);
    
    if ( r_checker > 0 || w_checker > 0 )
     out += text[lang].kings.format(r_checker, w_checker); 
    
    update_status(out, 'ok');          
}


function draw(){
    context.clearRect(0, 0, canvas.width, canvas.height);

    for(var i=0; i<tiles.length; i++){
        tiles[i].draw(context);
    }

    for(var i=0; i<tiles.length; i++){
        tiles[i].drawPiece(context);
    }
}

function collide( source, target ) {
    return !(
        ( ( source.y + source.h ) < ( target.y ) ) ||
        ( source.y > ( target.y + target.h ) ) ||
        ( ( source.x + source.w ) < target.x ) ||
        ( source.x > ( target.x + target.w ) )
    );
}

canvas.onmouseup = function(e){	
	
    moving = false;
    var mp = getPos(canvas, e);
    canvas.style.cursor = 'default';

    if(BOARD_DEF.move == AI && withAI){
        return;
    }
    
    if (stopLoop == true)
        return;	
    
    var donePlayerMove = false;
    for(var i=0; i<tiles.length; i++){    	
        var t = {x: tiles[i].x, y: tiles[i].y, w: tiles[i].size, h: tiles[i].size};
        if(collide(mp, t) && piecemoving != null && tiles[i] != piecemoving && tiles[i].sqr != SQR.NONE)
         {           	
        	if(piecemoving.piece == null) return;
            var move = piecemoving.piece.move(piecemoving.sqr, tiles[i].sqr);
            if(move == null) return;
            if(!updatePiecesPos(piecemoving, move, i, BOARD_DEF)) break;           
            donePlayerMove = true;            
            break;
        }
    }

    if(piecemoving != null && piecemoving.piece != null && !donePlayerMove){
        piecemoving.piece.x = piecemoving.x;
        piecemoving.piece.y = piecemoving.y;
        piecemoving.piece.size = piecemoving.size;
        piecemoving = null;
    }

}

function updatePiecesPos(piecemoving, move, i, bf){
    if(!move[0]) return false;

    switch(move[1])
    {
        case MOVE_TYPE.MOVE_NORMAL:
        {
            tiles[i].piece = piecemoving.piece;
            tiles[i].piece.x = tiles[i].x;
            tiles[i].piece.y = tiles[i].y;
            tiles[i].piece.size = tiles[i].size;
            piecemoving.piece = null;
            piecemoving = null;

            updateSuperPiece(bf.rPieces, PIECE_TYPE.SUPER_RED);
            updateSuperPiece(bf.yPieces, PIECE_TYPE.SUPER_YELLOW);

            break;
        }
        case MOVE_TYPE.MOVE_CAPTURE:
        {
            tiles[i].piece = piecemoving.piece;
            tiles[i].piece.x = tiles[i].x;
            tiles[i].piece.y = tiles[i].y;
            tiles[i].piece.size = tiles[i].size;

            for(var toRemove of tiles){
                if (toRemove.sqr == move[2]){
                    toRemove.piece = null;
                    break;
                }
            }

            piecemoving.piece = null;
            piecemoving = null;

            updateSuperPiece(bf.rPieces, PIECE_TYPE.SUPER_RED);
            updateSuperPiece(bf.yPieces, PIECE_TYPE.SUPER_YELLOW);
        }
    }

    updateStones ();
        
    return true;
}

function updateSuperPiece(piecePos, pieceType){
    for(var tile of tiles){
        for(var rP of piecePos){
            if(BOARD_DEF.board[rP] == pieceType && tile.sqr == rP){
                tile.piece.pieceType = pieceType;
                break;
            }
        }
    }
 }

canvas.onmousedown =  function(e){

	canvas.style.cursor = 'not-allowed';
    if(BOARD_DEF.move == AI && withAI){
        return;
    }

    var mp = getPos(canvas, e);      

    for(var i=0; i<tiles.length; i++){
        var t = {x: tiles[i].x, y: tiles[i].y, w: tiles[i].size, h: tiles[i].size};
        if(collide(mp, t) && tiles[i].piece != null && tiles[i].sqr != SQR.NONE){

            var pTypeSuper = tiles[i].piece.pieceType == PIECE_TYPE.SUPER_RED ? PLAYER.P1 : PLAYER.P2;

            if(BOARD_DEF.move != tiles[i].piece.pieceType){
                if(pTypeSuper != BOARD_DEF.move)
                    break;
            }
                      
            for(var moves of BOARD_DEF.availableMoves){            	
                if(tiles[i].sqr == moves.piece){
                    piecemoving = tiles[i];
                    mp.size = tilesize;  
                    canvas.style.cursor = 'pointer';
                    piecemoving.piece.mouseMove(mp);                    
                    moving = true;
                    break;
                }
            }
            break;
        }
    }
}

canvas.onmousemove = function(e){

    if(moving && piecemoving != null){    	
    	canvas.style.cursor = 'pointer';
        var pos = getPos(canvas, e);
        pos.size = tilesize;
        piecemoving.piece.mouseMove(pos);        
    }
}

function getPos(c, e){
    var rect = c.getBoundingClientRect();
    return {x:e.clientX - rect.left, y: e.clientY - rect.top, h: 1, w:1};
}

initBoard(BOARD_DEF);
init();

if(DEBUG){
    var zz = document.createElement('div');
    document.body.append(zz);
}

var counterAlert = 0;
var aiMoveCounter = 0;
var stopLoop = false;

function update_status (msg, type)
{
  	var status = document.getElementById('status');
  	status.innerHTML = msg;
  	
  	if (type == 'init')
  	{	
  	 status.style.color = 'green'
  	 status.classList.remove('blinking');	 
  	}  	
  	else if (type == 'blink')
  	  status.classList.add('blinking'); 
}


function loop(time) {
    if(DEBUG){
        zz.innerHTML = "";
        zz.innerHTML = printBoard();
    }

    if(withAI && BOARD_DEF.move == AI){
        aiMoveCounter++;
        if(aiMoveCounter > 20){
            if(BOARD_DEF.availableMoves.length > 0) {
                aiMove(BOARD_DEF);
                aiMoveCounter = 0;
            }
        }
     }

    draw();     	
    
    if(BOARD_DEF.availableMoves.length <= 0){
        counterAlert++;
    }
    if(counterAlert > 30){
        var p = BOARD_DEF.move == PLAYER.P1 ? text[lang].White : text[lang].Red;
        stopLoop = true;
        update_status( p +text[lang].wins, 'blink');
    }    
   	    	
   	if  ( result_draw )
	 {
   	      stopLoop = true;
          update_status( text[lang].draw, 'blink');
       	  blink_result(); 
	 }
 
    
    if(!stopLoop)
     requestAnimationFrame(loop);
}

requestAnimationFrame(loop);